package mrtjp.projectred.exploration;

import java.util.Random;

import net.minecraft.init.Blocks;
import net.minecraft.block.Block;
import net.minecraft.block.BlockStoneBrick;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import gcewing.codechicken.lib.vec.BlockCoord;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

public class BlockPhotosyntheticStoneBrick extends BlockStoneBrick
{
    public static final String[] STONE_BRICK_TYPES = new String[] { "default", "mossy", "cracked", "chiseled" };
    public static final String[] names = new String[] { null, "mossy", "cracked", "carved" };
    @SideOnly(Side.CLIENT)
    private IIcon[] icons;

    public BlockPhotosyntheticStoneBrick()
    {
        setHardness(1.5F);
        setResistance(10.0F);
        setStepSound(soundTypePiston);
        setBlockName("stonebricksmooth");
        setBlockTextureName("stonebrick");
        setTickRandomly(true);
    }

    @Override
    public void updateTick(World w, int x, int y, int z, Random ran)
    {
        switch (w.getBlockMetadata(x, y, z)) {
        case 0:
            crackFromHeat(w, x, y, z, ran);
            return;
        case 1:
            spreadMossToNearby(w, x, y, z, ran);
            return;
        }
    }

    public void crackFromHeat(World w, int x, int y, int z, Random ran)
    {
        BlockCoord bc = new BlockCoord(x, y, z);
        if (isBlockWet(w, bc) && isBlockHot(w, bc))
            if (ran.nextInt(3) == 0)
                w.setBlock(x, y, z, Blocks.stonebrick, 2, 3);
    }

    public void spreadMossToNearby(World w, int x, int y, int z, Random ran)
    {
        if (!w.isAirBlock(x, y + 1, z) || w.canBlockSeeTheSky(x, y + 1, z))
            return;
        for (int i = 0; i < 6; i++)
        {
            BlockCoord bc = new BlockCoord(x, y, z).offset(i);
            Block block = w.getBlock(bc.x, bc.y, bc.z);
            int meta = w.getBlockMetadata(bc.x, bc.y, bc.z);
            if (!w.isAirBlock(bc.x, bc.y + 1, bc.z) || w.canBlockSeeTheSky(bc.x, bc.y + 1, bc.z))
                continue;
            if (block == Blocks.cobblestone)
            {
                if (isBlockWet(w, bc))
                    if (ran.nextInt(3) == 0)
                        w.setBlock(bc.x, bc.y, bc.z, Blocks.mossy_cobblestone, 0, 3);
            }
            else if (block == Blocks.stonebrick && meta == 2)
                if (isBlockWet(w, bc))
                    if (ran.nextInt(3) == 0)
                        w.setBlock(bc.x, bc.y, bc.z, Blocks.stonebrick, 1, 3);
        }
    }

    public boolean isBlockWet(World w, BlockCoord b)
    {
        for (int i = 0; i < 6; i++)
        {
            BlockCoord bc = b.copy().offset(i);
            Block block = w.getBlock(bc.x, bc.y, bc.z);
            if (block == Blocks.flowing_water || block == Blocks.water)
                return true;
        }
        return false;
    }

    public boolean isBlockHot(World w, BlockCoord b)
    {
        for (int i = 0; i < 6; i++)
        {
            BlockCoord bc = b.copy().offset(i);
            Block block = w.getBlock(bc.x, bc.y, bc.z);
            if (block == Blocks.flowing_lava || block == Blocks.lava)
                return true;
        }
        return false;
    }

    @Override
    public void registerBlockIcons(IIconRegister reg)
    {
        super.registerBlockIcons(reg);
        icons = new IIcon[names.length];
        //Block.stoneBrick.registerIcons(reg); //?
        for (int i = 0; i < this.icons.length; ++i)
        {
            String s = this.getTextureName();
            if (names[i] != null)
                s = s + "_" + names[i];
            this.icons[i] = reg.registerIcon(s);
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public IIcon getIcon(int par1, int par2)
    {
        if (icons == null)
            icons = new IIcon[names.length];
        if (par2 < 0 || par2 >= names.length)
            par2 = 0;
        return this.icons[par2];
    }
}
