package mrtjp.projectred.exploration

import codechicken.microblock.BlockMicroMaterial
import cpw.mods.fml.common.network.IGuiHandler
import cpw.mods.fml.common.registry.GameRegistry
import cpw.mods.fml.relauncher.{Side, SideOnly}
import mrtjp.projectred.core.ReflectionUtils
import mrtjp.projectred.ProjectRedExploration
import mrtjp.projectred.ProjectRedExploration._
import mrtjp.projectred.core.{PRColors, Configurator, IProxy}
import mrtjp.projectred.exploration.BlockOre.EnumOre
import mrtjp.projectred.exploration.BlockSpecialStone.EnumSpecialStone
import mrtjp.projectred.exploration.BlockStainedLeaf.EnumDyeTrees
import mrtjp.projectred.exploration.ItemGemSaw.GemSawItemRenderer
import net.minecraft.init.Blocks
import net.minecraft.block.Block
import net.minecraft.entity.player.EntityPlayer
import net.minecraft.world.World
import net.minecraftforge.client.MinecraftForgeClient
import net.minecraftforge.common.MinecraftForge
import net.minecraftforge.common.util.EnumHelper
import net.minecraftforge.oredict.OreDictionary

class ExplorationProxy_server extends IProxy with IGuiHandler
{
    override def preinit() {
        if (Configurator.retroGeneration) RetroGenerationManager.registerRetroGenerators()
        else GameRegistry.registerWorldGenerator(GenerationManager.instance, 0)

        itemWoolGin = new ItemWoolGin //(Configurator.item_woolginID.getInt)
        itemBackpack = new ItemBackpack //(Configurator.item_backpackID.getInt)

        blockOres = new BlockOre //(Configurator.block_oresID.getInt)
        GameRegistry.registerBlock(blockOres, classOf[ItemBlockOre], "projectred.exploration.ore")
        for (o <- EnumOre.VALID_ORES) blockOres.setHarvestLevel("pickaxe", o.harvesLevel, o.ordinal())

        blockStones = new BlockSpecialStone //(Configurator.block_stonesID.getInt)
        GameRegistry.registerBlock(blockStones, classOf[ItemBlockSpecialStone], "projectred.exploration.stone")

        blockStoneWalls = new BlockSpecialStoneWall //(Configurator.block_stoneWallsID.getInt)
        GameRegistry.registerBlock(blockStoneWalls, classOf[ItemBlockSpecialStoneWalls], "projectred.exploration.stonewalls")

        blockStainedLeaf = new BlockStainedLeaf //(Configurator.block_stainedLeafID.getInt)
        GameRegistry.registerBlock(blockStainedLeaf, classOf[ItemBlockMetaHandler], "projectred.exploration.dyeleaf")

        blockStainedSapling = new BlockStainedSapling //(Configurator.block_stainedSaplingID.getInt)
        GameRegistry.registerBlock(blockStainedSapling, classOf[ItemBlockStainedSapling], "projectred.exploration.dyesapling")
        for (i <- 0 until 16) OreDictionary.registerOre(PRColors.get(i).getOreDict, EnumDyeTrees.VALID_FOLIAGE(i).getSappling)

        if (Configurator.gen_SpreadingMoss.getBoolean(true))
        {
//            val mc = Block.cobblestoneMossy.blockID
//            Block.blocksList(mc) = null
//            new BlockPhotosyntheticCobblestone(mc)
//            val sb = Block.stoneBrick.blockID
//            Block.blocksList(sb) = null
//            new BlockPhotosyntheticStoneBrick(sb)
             replaceVanillaBlock("mossy_cobblestone", "mossy_cobblestone", "field_150341_Y", new BlockPhotosyntheticCobblestone())
             replaceVanillaBlock("stonebrick", "stonebrick", "field_150417_aV", new BlockPhotosyntheticStoneBrick())
        }

        toolMaterialRuby = EnumHelper.addToolMaterial("RUBY", 2, 500, 8.0F, 4, 12)
        toolMaterialSapphire = EnumHelper.addToolMaterial("SAPPHIRE", 2, 500, 8.0F, 3, 16)
        toolMaterialPeridot = EnumHelper.addToolMaterial("PERIDOT", 2, 500, 8.75F, 3.25F, 12)

        itemRubyAxe = new ItemGemAxe(EnumSpecialTool.RUBYAXE)
        itemSapphireAxe = new ItemGemAxe(EnumSpecialTool.SAPPHIREAXE)
        itemPeridotAxe = new ItemGemAxe(EnumSpecialTool.PERIDOTAXE)
        itemRubyAxe.setHarvestLevel("axe", 2)
        itemSapphireAxe.setHarvestLevel("axe", 2)
        itemPeridotAxe.setHarvestLevel("axe", 2)

        itemRubyHoe = new ItemGemHoe(EnumSpecialTool.RUBYHOE)
        itemSapphireHoe = new ItemGemHoe(EnumSpecialTool.SAPPHIREHOE)
        itemPeridotHoe = new ItemGemHoe(EnumSpecialTool.PERIDOTHOE)
        itemRubyHoe.setHarvestLevel("hoe", 2)
        itemSapphireHoe.setHarvestLevel("hoe", 2)
        itemPeridotHoe.setHarvestLevel("hoe", 2)

        itemRubyPickaxe = new ItemGemPickaxe( EnumSpecialTool.RUBYPICKAXE)
        itemSapphirePickaxe = new ItemGemPickaxe(EnumSpecialTool.SAPPHIREPICKAXE)
        itemPeridotPickaxe = new ItemGemPickaxe(EnumSpecialTool.PERIDOTPICKAXE)
        itemRubyPickaxe.setHarvestLevel("pickaxe", 2)
        itemSapphirePickaxe.setHarvestLevel("pickaxe", 2)
        itemPeridotPickaxe.setHarvestLevel("pickaxe", 2)

        itemRubyShovel = new ItemGemShovel(EnumSpecialTool.RUBYSHOVEL)
        itemSapphireShovel = new ItemGemShovel(EnumSpecialTool.SAPPHIRESHOVEL)
        itemPeridotShovel = new ItemGemShovel(EnumSpecialTool.PERIDOTSHOVEL)

        itemRubyShovel.setHarvestLevel("shovel", 2)
        itemSapphireShovel.setHarvestLevel("shovel", 2)
        itemPeridotShovel.setHarvestLevel("shovel", 2)

        itemRubySword = new ItemGemSword(EnumSpecialTool.RUBYSWORD)
        itemSapphireSword = new ItemGemSword(EnumSpecialTool.SAPPHIRESWORD)
        itemPeridotSword = new ItemGemSword(EnumSpecialTool.PERIDOTSWORD)

        itemRubySword.setHarvestLevel("sword", 2)
        itemSapphireSword.setHarvestLevel("sword", 2)
        itemPeridotSword.setHarvestLevel("sword", 2)

        itemGoldSaw = new ItemGemSaw(EnumSpecialTool.GOLDSAW)
        itemRubySaw = new ItemGemSaw(EnumSpecialTool.RUBYSAW)
        itemSapphireSaw = new ItemGemSaw(EnumSpecialTool.SAPPHIRESAW)
        itemPeridotSaw = new ItemGemSaw(EnumSpecialTool.PERIDOTSAW)

        itemWoodSickle = new ItemGemSickle(EnumSpecialTool.WOODSICKLE)
        itemStoneSickle = new ItemGemSickle(EnumSpecialTool.STONESICKLE)
        itemIronSickle = new ItemGemSickle(EnumSpecialTool.IRONSICKLE)
        itemGoldSickle = new ItemGemSickle(EnumSpecialTool.GOLDSICKLE)
        itemRubySickle = new ItemGemSickle(EnumSpecialTool.RUBYSICKLE)
        itemSapphireSickle = new ItemGemSickle(EnumSpecialTool.SAPPHIRESICKLE)
        itemPeridotSickle = new ItemGemSickle(EnumSpecialTool.PERIDOTSICKLE)
        itemDiamondSickle = new ItemGemSickle(EnumSpecialTool.DIAMONDSICKLE)

        for (s <- EnumSpecialStone.VALID_STONE) BlockMicroMaterial.createAndRegister(ProjectRedExploration.blockStones, s.meta)
    }

    override def init()
    {
    }

    override def postinit()
    {
        ExplorationRecipes.initOreDict()
        ExplorationRecipes.initRecipes()
    }

    final val ID_Bag = 1

    def getServerGuiElement(ID:Int, player:EntityPlayer, world:World, x:Int, y:Int, z:Int):AnyRef =
    {
        if (ID == ID_Bag)
        {
            val held = player.getHeldItem
            if (held.getItem == ProjectRedExploration.itemBackpack) return ItemBackpack.getContainer(player)
        }
        null
    }

    def getClientGuiElement(ID:Int, player:EntityPlayer, world:World, x:Int, y:Int, z:Int):AnyRef =
    {
        if (ID == ID_Bag)
        {
            val held = player.getHeldItem
            if (held.getItem == ProjectRedExploration.itemBackpack) return new GuiBackpack(player, ItemBackpack.getBackpackInventory(player), held)
        }
        null
    }

    override def version = "@VERSION@"
    override def build = "@BUILD_NUMBER@"
    
    def replaceVanillaBlock(blockName: String, fieldName: String, srgFieldName: String, newBlock: Block) {
        val f = ReflectionUtils.getFinalField(classOf[Blocks], fieldName, srgFieldName);
        val oldBlock = f.get(null).asInstanceOf[Block];
        val id = Block.getIdFromBlock(oldBlock);
        Block.blockRegistry.addObject(id, blockName, newBlock);
        f.set(null, newBlock);
    }
}

class ExplorationProxy_client extends ExplorationProxy_server
{
    @SideOnly(Side.CLIENT)
    override def init()
    {
        super.init()
        MinecraftForgeClient.registerItemRenderer(itemGoldSaw, GemSawItemRenderer.instance)
        MinecraftForgeClient.registerItemRenderer(itemRubySaw, GemSawItemRenderer.instance)
        MinecraftForgeClient.registerItemRenderer(itemSapphireSaw, GemSawItemRenderer.instance)
        MinecraftForgeClient.registerItemRenderer(itemPeridotSaw, GemSawItemRenderer.instance)
    }
}

object ExplorationProxy extends ExplorationProxy_client
